/****************************************************************************
** Date:2006.6.30
** Author: $BCf@>MN5.(B
** Comment: RGB Convert
*****************************************************************************/

#include <math.h>
#include "level1.h"
#include "convert.h"

static void chrgb_n(RGB *,int);
static void bright_n(RGB *,int,int);
static void gamma_n(RGB *,double,int);
static void chmono_n(RGB * , int );

// RGB$B$rF~$l49$($k4X?t(B---------------------------

void chrgb(imElement *im,int depth)
{
  int i;
  RGB *rgb;
  rgb = (RGB *)im->xim->data;
  
  for(i=0; i < im->width * im->height; i++)
    {
      chrgb_n(rgb,depth);
      rgb++;
    }
}

// RGB$BJQ49$N2<0L4X?t(B---------------------------

static void chrgb_n(RGB *rgb,int depth)
{
  RGB temp;
  if(depth == 16){

    // bitmap $B$r(B 24 bit $B$KLa$9(B

    temp.red   = (rgb->green)>>3;
    temp.green = (rgb->green & 0x07)<<2 | (rgb->blue)>>6;
    temp.blue  = (rgb->blue & 0x1F);
    
    rgb->red   = 0x00;
    rgb->green = (temp.green<<3 | temp.blue >> 2);
    rgb->blue  = (temp.blue & 0x03) << 6 | temp.red;
  }else{
    temp.red   = rgb->red;
    temp.green = rgb->green;
    temp.blue  = rgb->blue;
    
    rgb->red   = temp.green; 
    rgb->green = temp.blue;
    rgb->blue  = temp.red;
  }
}

//$B51EY$rJQ49$9$k4X?t(B---------------------------

void bright(imElement *im,int light,int depth)
{
  int i;
  RGB *rgb;
    
  rgb = (RGB *)im->xim->data;
  
  for(i=0;i < im->width *im->height;i++)
    {
      bright_n(rgb,light,depth);
      rgb++;
    }
}


//$B51EYJQ49$N2<0L4X?t(B---------------------------

static void bright_n(RGB *rgb,int light,int depth)
{
  int r,g,b;

  // 16bitmap $B$N;~$N=hM}(B

  if(depth == 16){

    // bitmap $B$r(B 24 bit $B$KLa$9(B---------------------------

    r  = (rgb->green)>>3;
    g  = (rgb->green & 0x07)<<2 | (rgb->blue)>>6;
    b  = (rgb->blue & 0x1F);
    
    if((r * light / 100)< 0x20)
      rgb->red   = r * light / 100;     
    else
      rgb->red = 0x1F;
   
    if((g * light / 100)< 0x20) 
      rgb->green = g * light / 100;
    else
      rgb->green = 0x1F;

    if((b * light / 100) < 0x20)
      rgb->blue  = b * light / 100;  
    else
      rgb->blue = 0x1F;
    
    rgb->blue  = (rgb->green & 0x03) << 6 | rgb->blue;
    rgb->green = (rgb->red<<3 | rgb->green >> 2);
    rgb->red   = 0x00;
    
    // 24bitmap $B$N;~$N=hM}(B---------------------------

  }else{ 
    r = rgb->red;
    g = rgb->green;
    b = rgb->blue;
    
    if((r * light / 100) < 0x100)
      rgb->red   = r * light / 100;
    else
      rgb->red = 0xFF;
    
    if((g * light / 100) < 0x100)
      rgb->green = g * light / 100;
    else
      rgb->green = 0xFF;
    
    if((b * light / 100) < 0x100)
      rgb->blue  = b * light / 100;  
    else
      rgb->blue = 0xFF;
  }
}  

//$B%,%s%^JQ49$r9T$&4X?t(B---------------------------

void chgamma(imElement *im,double r,int depth)
{
  int i;
  RGB *rgb;
  rgb = (RGB *)im->xim->data;
  
  for(i=0;i < im->width *im->height;i++)
    {
      gamma_n(rgb,r,depth);
      rgb++;
    }
}

//$B%,%s%^JQ49$r9T$&2<0L4X?t(B---------------------------

static void gamma_n(RGB *rgb,double gamm,int depth)
{
  unsigned int r,g,b;
  
  // 24bitmap $B$N;~$N=hM}(B---------------------------

  if(depth == 16){

    // bitmap $B$r(B 24 bit $B$KLa$9!#(B---------------------------

    r  = (rgb->green)>>3;
    g  = (rgb->green & 0x07)<<2 | (rgb->blue)>>6;
    b  = (rgb->blue & 0x1F);
    
    rgb->blue  = (rgb->green & 0x03) << 6 | rgb->blue;
    rgb->green = (rgb->red<<3 | rgb->green >> 2);
    rgb->red   = 0x00;
    
    // 24bitmap $B$N;~$N=hM}(B---------------------------

  }else{ 
    r = rgb->red;
    g = rgb->green;
    b = rgb->blue;
    
    if((rgb->red = (unsigned char)(pow(rgb->red / 255.0,1.0/gamm) * 255.0)) < 0x100);     
    else
      rgb->red   = 0xFF;
    
    if(( rgb->green = (unsigned char)(pow(rgb->green / 255.0,1.0/gamm) * 255.0)) < 0x100);
    else
      rgb->green = 0xFF;
    
    if((rgb->blue  = (unsigned char)(pow(rgb->blue  / 255.0,1.0/gamm)) * 255.0) < 0x100);     
    else
      rgb->blue   = 0xFF;
  }
} 

//$B%b%N%/%mJQ49$r9T$&4X?t(B---------------------------

void chmono(imElement *im,int depth)
{
  int i;
  RGB *rgb;
  rgb = (RGB *)im->xim->data;
  
  for(i=0;i < im->width *im->height;i++)
    {
      chmono_n(rgb,depth);
      rgb++;
    }
}


//$B%b%N%/%mJQ49$r9T$&2<0L4X?t(B---------------------------

static void chmono_n(RGB *rgb ,int depth)
{
  unsigned char m;
  
  if(depth == 16){
    m = (((rgb->green)>>3) + ((rgb->green & 0x07)<<2 | (rgb->blue)>>6) + (rgb->blue & 0x1F))/3;
    
    rgb->red   = 0x00;
    rgb->green = (m << 3 | m >>2); 
    rgb->blue  = ((m & 0x03) << 6 | m);
  }else{
    m = (rgb->red + rgb->green + rgb->blue)/3;
    rgb->red   = m;
    rgb->green = m;
    rgb->blue  = m;
  }
}

// $BFs$D$N?'$NJ?6Q$r5a$a$k$?$a$N4X?t(B---------------------------
 
void aveColor(RGB *dst ,RGB *src1,RGB *src2)
{
  dst->red   = (src1->red   + src2->red  )/2;
  dst->green = (src1->green + src2->green)/2;
  dst->blue  = (src1->blue  + src2->blue )/2;
}

void HSV2RGB(RGB *rgb,int H, double S, double V)
{
  char in;
  double fl;
  
  in = H/60;
  fl = ((double)H/60) - in;
}
